require "import"
import "android.speech.RecognizerIntent"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.net.Uri"
import "android.content.Intent"
import "android.os.Build"
import "android.os.VibrationEffect"
import "android.os.Vibrator"
import "android.content.Context"
import "java.util.Locale"

-- activity define if not available in your environment
local Activity = luajava.bindClass("android.app.Activity")
local activity = service or Activity:getApplication()

local spokenText = ""

function saveWords(words)
    spokenText = words
end

function vibrate(duration)
    local vibrator = service.getSystemService(Context.VIBRATOR_SERVICE)
    if Build.VERSION.SDK_INT >= 26 then
        vibrator.vibrate(VibrationEffect.createOneShot(duration, VibrationEffect.DEFAULT_AMPLITUDE))
    else
        vibrator.vibrate(duration)
    end
end

function startListening()
    vibrate(300) -- Vibrate when starting listening

    local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "ne-NP") -- Nepali language code
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, activity.getPackageName())

    local speechRecognizer = SpeechRecognizer.createSpeechRecognizer(activity)

    local speechListener = luajava.createProxy("android.speech.RecognitionListener", {
        onResults = function(results)
            local data = results.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            if data ~= nil and data.size() > 0 then
                local recognizedText = data.get(0)
                saveWords(recognizedText)

                -- Translate using Google Translate API
                local url = "https://translate.google.com/m?sl=auto&hl=" .. Locale.getDefault().getLanguage() .. "&q=" .. Uri.encode(recognizedText) .. "&tl=en"

                Http.get(url, function(status, result)
                    if status == 200 then
                        local translatedWords = result:match('<div class="result%-container">([^<]+)</div>')
                        if translatedWords then
                            service.insertText(service.getEditText(), translatedWords .. " ")
                            this.speak(translatedWords) -- Speak out the translated text
                        end
                    end
                end)
            end
            vibrate(300) -- Vibrate when speech recognition ends
        end,
        onError = function(errorCode)
            vibrate(300) -- Vibrate on error
        end
    })

    speechRecognizer.setRecognitionListener(speechListener)
    speechRecognizer.startListening(recognizerIntent)
end

startListening()
return true